const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

function BaseSearch(){}

BaseSearch.prototype = {
	startSearch: function(searchString, searchParam, previousResult, listener) {
		var defaultIndex = 0;
		var matchCount = 0;
		var searchResult = 0;
		this.searchSite = "";

		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var browserWindow = wm.getMostRecentWindow("navigator:browser");
		this.searchString = browserWindow.gURLBar.value;

		this.searchPhrase = this.searchString;
		this.searchPage = 0;

		this._results = []
		this._listener = listener;

		this.init();
		this._execute();
	},

	stopSearch: function(){

	},

	_execute: function(event){
		this._results = [];
		if(this.execute()){
			this.matchCount = this._results.length;
			this.searchResult = Ci.nsIAutoCompleteResult.RESULT_SUCCESS;
			this._listener.onSearchResult(this, this);
		} else{
			this.matchCount = 0;
			this.searchResult = Ci.nsIAutoCompleteResult.RESULT_FAILURE;
			this._results = [];
			this._listener.onSearchResult(this, this);
		}
	},

	getCommentAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].comment;
	},

	getImageAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].image;
	},

	getStyleAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].style;
	},

	getValueAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].value;
	},

	removeValueAt: function(index, removeFromDb){ },

	QueryInterface: XPCOMUtils.generateQI([Ci.nsIAutoCompleteSearch, Ci.nsIAutoCompleteResult])
};

IndexSearch = function(){ };
IndexSearch.prototype = new BaseSearch();
IndexSearch.prototype.classDescription = 'URLbar support for bookmarkTools';
IndexSearch.prototype.contractID = '@mozilla.org/autocomplete/search;1?name=cl-index';
IndexSearch.prototype.classID = Components.ID("4fa39240-a28a-11de-8a39-0800200c9a66");

IndexSearch.prototype.getFileObject = function(){
	var file = null;
	
	var prefInstance = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch);
	
	if(prefInstance.getBoolPref("bookmarkTools.indexPathDefault")){
		file = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties)
			.get("ProfD", Components.interfaces.nsIFile);
	}else {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefService).
                      getBranch("bookmarkTools.");
		file =  prefs.getComplexValue("indexPath", Components.interfaces.nsILocalFile);
	}
	return file;
}

IndexSearch.prototype.getSearchDir = function(){
	var file = this.getFileObject();

	file.append("cl_index");
	file.append("search");
	
	if(!file.exists() || !file.isDirectory()) 
		file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);

	return file.clone();
}

IndexSearch.prototype.init = function(){
	var instReader = Components.classes["@bookmarktools.mozdev.org/search/indexReader;1"].createInstance();
	this.indexReader = instReader.QueryInterface(Components.interfaces.ICLReader);

	this.indexReader.setPath(this.getSearchDir().path);
}

IndexSearch.prototype.search = function(size){
	if(this.indexReader.exists()){
	
		this.searchString = this.trim(this.searchString);
	
		if(this.searchString && this.searchString.length > 2){
			var array = Components.classes["@mozilla.org/array;1"]
			.createInstance(Components.interfaces.nsIMutableArray);
			
			this.indexReader.search(this.searchString, array, 0, size);

			return array;
		}
	}
}

IndexSearch.prototype.truncate = function(str, size, shortcut){
	if(str.length < size)
		return str;
		
	var shortStr = str.substring(0, size);
	return shortStr + shortcut;
}

IndexSearch.prototype.trim = function(str) {
	return str.replace(/^\s+/, '').replace(/\s+$/, '');
}

IndexSearch.prototype.execute = function(){
	try {
	
		var prefInstance = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch);
	
		var size = prefInstance.getIntPref("bookmarkTools.addressbar");
		if(size > 0){
			var array = this.search(size);
			var e = array.enumerate();
		
			var bmsvc = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
			.getService(Components.interfaces.nsINavBookmarksService);

			var ioService = Components.classes["@mozilla.org/network/io-service;1"]
			.getService(Components.interfaces.nsIIOService);
			
			var faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
			.getService(Components.interfaces.nsIFaviconService);
								
			var truncateSize = 100;
			var truncateShortCut = " [..]";
			
			while (e.hasMoreElements()) {
				var resultNode = e.getNext().QueryInterface(Components.interfaces.IResultNode);
				var uri = ioService.newURI(resultNode.primaryKey, null, null);
				
				if(bmsvc.isBookmarked(uri)){
				
					var itemIDs = bmsvc.getBookmarkIdsForURI(uri, {});
					var iconUri = "chrome://browser/skin/Bookmarks-folder.png";
			
					try {
						iconUri = faviconService.getFaviconForPage(uri).spec;
					} catch(e) {
						iconUri = "chrome://browser/skin/Bookmarks-folder.png"
					}
				
					this._results.push({
						value: this.truncate(resultNode.primaryKey, truncateSize, truncateShortCut),
						comment: this.truncate(bmsvc.getItemTitle(itemIDs[0]), truncateSize, truncateShortCut),
						image: iconUri,
						style: 'suggesthint bookmarktools cl-index '
					});
				}
			}
		}
	}catch(e){
		return false;
	}
	return true;
}

function NSGetModule(compMgr, fileSpec) {
	return XPCOMUtils.generateModule([IndexSearch]);
}